@@ -1,5 +1,5 @@
 // SPDX-License-Identifier: GPL-2.0
-/* Copyright (C) 2007-2020  B.A.T.M.A.N. contributors:
+/* Copyright (C) B.A.T.M.A.N. contributors:
  *
  * Andreas Langer <an.langer@gmx.de>, Marek Lindner <mareklindner@neomailbox.ch>
  *
@@ -43,7 +43,6 @@
 #include "bat-hosts.h"
 #include "sys.h"
 #include "debug.h"
-#include "debugfs.h"
 #include "netlink.h"
 
 #define PATH_BUFF_LEN 400
@@ -118,18 +117,8 @@ int file_exists(const char *fpath)
 	return stat(fpath, &st) == 0;
 }
 
-static void file_open_problem_dbg(const char *dir, const char *full_path)
+static void file_open_problem_dbg(const char *full_path)
 {
-	struct stat st;
-
-	if (strstr(dir, "/sys/")) {
-		if (stat("/sys/", &st) != 0) {
-			fprintf(stderr, "Error - the folder '/sys/' was not found on the system\n");
-			fprintf(stderr, "Please make sure that the sys filesystem is properly mounted\n");
-			return;
-		}
-	}
-
 	if (!file_exists(module_ver_path)) {
 		fprintf(stderr, "Error - batman-adv module has not been loaded\n");
 		return;
@@ -140,14 +129,6 @@ static void file_open_problem_dbg(const char *dir, const char *full_path)
 	fprintf(stderr, "Consult the README if you wish to learn more about compiling options into batman-adv.\n");
 }
 
-static int str_is_mcast_addr(char *addr)
-{
-	struct ether_addr *mac_addr = ether_aton(addr);
-
-	return !mac_addr ? 0 :
-		mac_addr->ether_addr_octet[0] & 0x01;
-}
-
 static bool ether_addr_valid(const uint8_t *addr)
 {
 	/* no multicast address */
@@ -161,199 +142,36 @@ static bool ether_addr_valid(const uint8_t *addr)
 	return true;
 }
 
-static void print_inv_bool(char *line)
+int read_file(const char *full_path, int read_opt)
 {
-	if (!strncmp("enabled", line, strlen("enabled")))
-		printf("disabled\n");
-	else if (!strncmp("disabled", line, strlen("disabled")))
-		printf("enabled\n");
-	else
-		printf("%s", line);
-}
-
-int read_file(const char *dir, const char *fname, int read_opt,
-	      float orig_timeout, float watch_interval, size_t header_lines)
-{
-	struct ether_addr *mac_addr;
-	struct bat_host *bat_host;
 	int res = EXIT_FAILURE;
-	float last_seen;
-	char full_path[500], *buff_ptr, *space_ptr, extra_char;
 	size_t len = 0;
 	FILE *fp = NULL;
-	size_t line;
 
-	if (read_opt & USE_BAT_HOSTS)
-		bat_hosts_init(read_opt);
-
-	snprintf(full_path, sizeof(full_path), "%s%s", dir, fname);
-
-open:
-	line = 0;
 	fp = fopen(full_path, "r");
-
 	if (!fp) {
 		if (!(read_opt & SILENCE_ERRORS))
-			file_open_problem_dbg(dir, full_path);
+			file_open_problem_dbg(full_path);
 
-		goto out;
+		return res;
 	}
 
-	if (read_opt & CLR_CONT_READ)
-		/* clear screen, set cursor back to 0,0 */
-		printf("\033[2J\033[0;0f");
-
-read:
 	while (getline(&line_ptr, &len, fp) != -1) {
-		if (line++ < header_lines && read_opt & SKIP_HEADER)
-			continue;
-
 		/* the buffer will be handled elsewhere */
 		if (read_opt & USE_READ_BUFF)
 			break;
 
-		/* skip timed out originators */
-		if (read_opt & NO_OLD_ORIGS)
-			if (sscanf(line_ptr, "%*s %f", &last_seen)
-			    && (last_seen > orig_timeout))
-				continue;
-
-		/* translation table: skip multicast */
-		if (line > header_lines &&
-		    read_opt & UNICAST_ONLY &&
-		    strlen(line_ptr) > strlen(" * xx:xx:xx:") &&
-		    str_is_mcast_addr(line_ptr+3))
-			continue;
-
-		/* translation table: skip unicast */
-		if (line > header_lines &&
-		    read_opt & MULTICAST_ONLY &&
-		    strlen(line_ptr) > strlen(" * xx:xx:xx:") &&
-		    !str_is_mcast_addr(line_ptr+3))
-			continue;
-
-		if (!(read_opt & USE_BAT_HOSTS)) {
-			if (read_opt & INVERSE_BOOL)
-				print_inv_bool(line_ptr);
-			else
-				printf("%s", line_ptr);
-
-			continue;
-		}
-
-		/* replace mac addresses with bat host names */
-		buff_ptr = line_ptr;
-
-		while ((space_ptr = strchr(buff_ptr, ' ')) != NULL) {
-
-			*space_ptr = '\0';
-			extra_char = '\0';
-
-			if (strlen(buff_ptr) == ETH_STR_LEN + 1) {
-				extra_char = buff_ptr[ETH_STR_LEN];
-				switch (extra_char) {
-				case ',':
-				case ')':
-					buff_ptr[ETH_STR_LEN] = '\0';
-					break;
-				default:
-					extra_char = '\0';
-					break;
-				}
-			}
-
-			if (strlen(buff_ptr) != ETH_STR_LEN)
-				goto print_plain_buff;
-
-			mac_addr = ether_aton(buff_ptr);
-
-			if (!mac_addr)
-				goto print_plain_buff;
-
-			bat_host = bat_hosts_find_by_mac((char *)mac_addr);
-
-			if (!bat_host)
-				goto print_plain_buff;
-
-			/* keep table format */
-			printf("%17s", bat_host->name);
-
-			goto written;
-
-print_plain_buff:
-			printf("%s", buff_ptr);
-
-written:
-			if (extra_char != '\0')
-				printf("%c", extra_char);
-
-			printf(" ");
-			buff_ptr = space_ptr + 1;
-		}
-
-		printf("%s", buff_ptr);
-	}
-
-	if (read_opt & CONT_READ) {
-		usleep(1000000 * watch_interval);
-		goto read;
-	}
-
-	if (read_opt & CLR_CONT_READ) {
-		if (fp)
-			fclose(fp);
-		usleep(1000000 * watch_interval);
-		goto open;
+		printf("%s", line_ptr);
 	}
 
 	if (line_ptr)
 		res = EXIT_SUCCESS;
 
-out:
-	if (fp)
-		fclose(fp);
-
-	if (read_opt & USE_BAT_HOSTS)
-		bat_hosts_free();
-
+	fclose(fp);
 	return res;
 }
 
-int write_file(const char *dir, const char *fname, const char *arg1,
-	       const char *arg2)
-{
-	int fd = -1, res = EXIT_FAILURE;
-	char full_path[500];
-	ssize_t write_len;
-
-	snprintf(full_path, sizeof(full_path), "%s%s", dir, fname);
-
-	fd = open(full_path, O_WRONLY);
-
-	if (fd < 0) {
-		file_open_problem_dbg(dir, full_path);
-		goto out;
-	}
-
-	if (arg2)
-		write_len = dprintf(fd, "%s %s", arg1, arg2);
-	else
-		write_len = write(fd, arg1, strlen(arg1) + 1);
-
-	if (write_len < 0) {
-		fprintf(stderr, "Error - can't write to file '%s': %s\n", full_path, strerror(errno));
-		goto out;
-	}
-
-	res = EXIT_SUCCESS;
-
-out:
-	if (fd >= 0)
-		close(fd);
-	return res;
-}
-
-struct ether_addr *translate_mac(const char *mesh_iface,
+struct ether_addr *translate_mac(struct state *state,
 				 const struct ether_addr *mac)
 {
 	struct ether_addr in_mac;
@@ -370,47 +188,16 @@ struct ether_addr *translate_mac(const char *mesh_iface,
 	if (!ether_addr_valid(in_mac.ether_addr_octet))
 		return mac_result;
 
-	translate_mac_netlink(mesh_iface, &in_mac, mac_result);
+	translate_mac_netlink(state, &in_mac, mac_result);
 
 	return mac_result;
 }
 
-int get_algoname(const char *mesh_iface, char *algoname, size_t algoname_len)
+int get_algoname(struct state *state, unsigned int mesh_ifindex,
+		 char *algoname, size_t algoname_len)
 {
-	char *path_buff;
-	int ret;
-
-	ret = get_algoname_netlink(mesh_iface, algoname, algoname_len);
-	if (ret != -EOPNOTSUPP)
-		return ret;
-
-	path_buff = malloc(PATH_BUFF_LEN);
-	if (!path_buff) {
-		fprintf(stderr, "Error - could not allocate path buffer: out of memory ?\n");
-		return -ENOMEM;
-	}
-
-	snprintf(path_buff, PATH_BUFF_LEN, SYS_ROUTING_ALGO_FMT, mesh_iface);
-	ret = read_file("", path_buff, USE_READ_BUFF | SILENCE_ERRORS, 0, 0, 0);
-	if (ret != EXIT_SUCCESS) {
-		ret = -ENOENT;
-		goto free_path_buf;
-	}
-
-	if (line_ptr[strlen(line_ptr) - 1] == '\n')
-		line_ptr[strlen(line_ptr) - 1] = '\0';
-
-	strncpy(algoname, line_ptr, algoname_len);
-	if (algoname_len > 0)
-		algoname[algoname_len - 1] = '\0';
-
-free_path_buf:
-	free(path_buff);
-
-	free(line_ptr);
-	line_ptr = NULL;
-
-	return ret;
+	return get_algoname_netlink(state, mesh_ifindex, algoname,
+				    algoname_len);
 }
 
 static int resolve_l3addr(int ai_family, const char *asc, void *l3addr)
@@ -555,10 +342,7 @@ static int resolve_mac_from_parse(struct nl_msg *msg, void *arg)
 	}
 
 err:
-	if (nl_arg->found)
-		return NL_STOP;
-	else
-		return NL_OK;
+	return NL_OK;
 }
 
 static struct ether_addr *resolve_mac_from_cache(int ai_family,
@@ -1064,45 +848,16 @@ int guess_netdev_type(const char *netdev, enum selector_prefix *type)
 	return -EINVAL;
 }
 
-static int check_mesh_iface_sysfs(struct state *state)
-{
-	char path_buff[PATH_BUFF_LEN];
-	DIR *dir;
-
-	/* try to open the mesh sys directory */
-	snprintf(path_buff, PATH_BUFF_LEN, SYS_BATIF_PATH_FMT,
-		 state->mesh_iface);
-
-	dir = opendir(path_buff);
-	if (!dir)
-		return -1;
-
-	closedir(dir);
-
-	return 0;
-}
-
 int check_mesh_iface(struct state *state)
 {
-	int ret;
-
 	state->mesh_ifindex = if_nametoindex(state->mesh_iface);
 	if (state->mesh_ifindex == 0)
 		return -1;
 
-	ret = check_mesh_iface_netlink(state->mesh_ifindex);
-	if (ret == 0)
-		return ret;
-
-	ret = check_mesh_iface_sysfs(state);
-	if (ret == 0)
-		return ret;
-
-	return -1;
+	return check_mesh_iface_netlink(state->mesh_ifindex);
 }
 
-static int check_mesh_iface_ownership_netlink(struct state *state,
-					      char *hard_iface)
+int check_mesh_iface_ownership(struct state *state, char *hard_iface)
 {
 	struct rtnl_link_iface_data link_data;
 	unsigned int hardif_index;
@@ -1121,49 +876,6 @@ static int check_mesh_iface_ownership_netlink(struct state *state,
 	return EXIT_SUCCESS;
 }
 
-static int check_mesh_iface_ownership_sysfs(struct state *state,
-					    char *hard_iface)
-{
-	char path_buff[PATH_BUFF_LEN];
-	int res;
-
-	/* check if this device actually belongs to the mesh interface */
-	snprintf(path_buff, sizeof(path_buff), SYS_MESH_IFACE_FMT, hard_iface);
-	res = read_file("", path_buff, USE_READ_BUFF | SILENCE_ERRORS, 0, 0, 0);
-	if (res != EXIT_SUCCESS) {
-		fprintf(stderr, "Error - the directory '%s' could not be read: %s\n",
-			path_buff, strerror(errno));
-		fprintf(stderr, "Is the batman-adv module loaded and sysfs mounted ?\n");
-		return EXIT_FAILURE;
-	}
-
-	if (line_ptr[strlen(line_ptr) - 1] == '\n')
-		line_ptr[strlen(line_ptr) - 1] = '\0';
-
-	if (strcmp(line_ptr, state->mesh_iface) != 0) {
-		fprintf(stderr, "Error - interface %s is part of batman network %s, not %s\n",
-			hard_iface, line_ptr, state->mesh_iface);
-		return EXIT_FAILURE;
-	}
-
-	return EXIT_SUCCESS;
-}
-
-int check_mesh_iface_ownership(struct state *state, char *hard_iface)
-{
-	int ret;
-
-	ret = check_mesh_iface_ownership_netlink(state, hard_iface);
-	if (ret == EXIT_SUCCESS)
-		return EXIT_SUCCESS;
-
-	ret = check_mesh_iface_ownership_sysfs(state, hard_iface);
-	if (ret == EXIT_SUCCESS)
-		return ret;
-
-	return EXIT_FAILURE;
-}
-
 static int get_random_bytes_syscall(void *buf __maybe_unused,
 				    size_t buflen __maybe_unused)
 {
