@@ -1,5 +1,5 @@
 // SPDX-License-Identifier: GPL-2.0
-/* Copyright (C) 2009-2020  B.A.T.M.A.N. contributors:
+/* Copyright (C) B.A.T.M.A.N. contributors:
  *
  * Andrew Lunn <andrew@lunn.ch>
  * Sven Eckelmann <sven@narfation.org>
@@ -40,12 +40,14 @@ static int originators_callback(struct nl_msg *msg, void *arg)
 	struct nlmsghdr *nlh = nlmsg_hdr(msg);
 	int last_seen_msecs, last_seen_secs;
 	struct print_opts *opts = arg;
+	char ifname_buf[IF_NAMESIZE];
 	struct bat_host *bat_host;
 	struct genlmsghdr *ghdr;
-	char ifname[IF_NAMESIZE];
+	uint32_t ifindex;
 	float last_seen;
 	uint8_t *neigh;
 	uint8_t *orig;
+	char *ifname;
 	char c = ' ';
 	uint8_t tq;
 
@@ -74,9 +76,16 @@ static int originators_callback(struct nl_msg *msg, void *arg)
 	orig = nla_data(attrs[BATADV_ATTR_ORIG_ADDRESS]);
 	neigh = nla_data(attrs[BATADV_ATTR_NEIGH_ADDRESS]);
 
-	if (!if_indextoname(nla_get_u32(attrs[BATADV_ATTR_HARD_IFINDEX]),
-			    ifname))
-		ifname[0] = '\0';
+	if (attrs[BATADV_ATTR_HARD_IFNAME]) {
+		ifname = nla_get_string(attrs[BATADV_ATTR_HARD_IFNAME]);
+	} else {
+		/* compatibility for Linux < 5.14/batman-adv < 2021.2 */
+		ifindex = nla_get_u32(attrs[BATADV_ATTR_HARD_IFINDEX]);
+		if (!if_indextoname(ifindex, ifname_buf))
+			ifname_buf[0] = '\0';
+
+		ifname = ifname_buf;
+	}
 
 	if (attrs[BATADV_ATTR_FLAG_BEST])
 		c = '*';
@@ -174,8 +183,7 @@ static int netlink_print_originators(struct state *state, char *orig_iface,
 
 	/* only parse routing algorithm name */
 	last_err = -EINVAL;
-	info_header = netlink_get_info(state->mesh_ifindex,
-				       BATADV_CMD_GET_ORIGINATORS, NULL);
+	info_header = netlink_get_info(state, BATADV_CMD_GET_ORIGINATORS, NULL);
 	free(info_header);
 
 	if (strlen(algo_name_buf) == 0)
@@ -196,8 +204,6 @@ static int netlink_print_originators(struct state *state, char *orig_iface,
 }
 
 static struct debug_table_data batctl_debug_table_originators = {
-	.debugfs_name = "originators",
-	.header_lines = 2,
 	.netlink_fn = netlink_print_originators,
 	.option_timeout_interval = 1,
 	.option_orig_iface = 1,
