@@ -57,7 +57,8 @@ static unsigned short dump_level_all = DUMP_TYPE_BATOGM | DUMP_TYPE_BATOGM2 |
 				       DUMP_TYPE_BATELP | DUMP_TYPE_BATICMP |
 				       DUMP_TYPE_BATUCAST | DUMP_TYPE_BATBCAST |
 				       DUMP_TYPE_BATUTVLV | DUMP_TYPE_BATFRAG |
-				       DUMP_TYPE_NONBAT | DUMP_TYPE_BATCODED;
+				       DUMP_TYPE_NONBAT | DUMP_TYPE_BATCODED |
+				       DUMP_TYPE_BATMCAST;
 static unsigned short dump_level;
 
 static void parse_eth_hdr(unsigned char *packet_buff, ssize_t buff_len, int read_opt, int time_printed);
@@ -82,6 +83,7 @@ static void tcpdump_usage(void)
 	fprintf(stderr, " \t\t%3d - batman unicast tvlv packets\n", DUMP_TYPE_BATUTVLV);
 	fprintf(stderr, " \t\t%3d - non batman packets\n", DUMP_TYPE_NONBAT);
 	fprintf(stderr, " \t\t%3d - batman coded packets\n", DUMP_TYPE_BATCODED);
+	fprintf(stderr, " \t\t%3d - batman multicast packets\n", DUMP_TYPE_BATMCAST);
 	fprintf(stderr, " \t\t%3d - batman ogm & non batman packets\n", DUMP_TYPE_BATOGM | DUMP_TYPE_NONBAT);
 }
 
@@ -101,7 +103,8 @@ static int print_time(void)
 	return 1;
 }
 
-static void batctl_tvlv_parse_gw_v1(void *buff, ssize_t buff_len)
+static void batctl_tvlv_parse_gw_v1(void *buff, ssize_t buff_len,
+				    int read_opt __maybe_unused)
 {
 	struct batadv_tvlv_gateway_data *tvlv = buff;
 	uint32_t down, up;
@@ -119,8 +122,9 @@ static void batctl_tvlv_parse_gw_v1(void *buff, ssize_t buff_len)
 	       down / 10, down % 10, up / 10, up % 10);
 }
 
-static void batctl_tvlv_parse_dat_v1(void (*buff)__attribute__((unused)),
-				     ssize_t buff_len)
+static void batctl_tvlv_parse_dat_v1(void *buff __maybe_unused,
+				     ssize_t buff_len,
+				     int read_opt __maybe_unused)
 {
 	if (buff_len != 0) {
 		fprintf(stderr, "Warning - dropping received %s packet as it is not the correct size (0): %zu\n",
@@ -131,8 +135,9 @@ static void batctl_tvlv_parse_dat_v1(void (*buff)__attribute__((unused)),
 	printf("\tTVLV DATv1: enabled\n");
 }
 
-static void batctl_tvlv_parse_nc_v1(void (*buff)__attribute__((unused)),
-				    ssize_t buff_len)
+static void batctl_tvlv_parse_nc_v1(void *buff __maybe_unused,
+				    ssize_t buff_len,
+				    int read_opt __maybe_unused)
 {
 	if (buff_len != 0) {
 		fprintf(stderr, "Warning - dropping received %s packet as it is not the correct size (0): %zu\n",
@@ -143,7 +148,8 @@ static void batctl_tvlv_parse_nc_v1(void (*buff)__attribute__((unused)),
 	printf("\tTVLV NCv1: enabled\n");
 }
 
-static void batctl_tvlv_parse_tt_v1(void *buff, ssize_t buff_len)
+static void batctl_tvlv_parse_tt_v1(void *buff, ssize_t buff_len,
+				    int read_opt __maybe_unused)
 {
 	struct batadv_tvlv_tt_data *tvlv = buff;
 	struct batadv_tvlv_tt_vlan_data *vlan;
@@ -183,7 +189,8 @@ static void batctl_tvlv_parse_tt_v1(void *buff, ssize_t buff_len)
 	}
 }
 
-static void batctl_tvlv_parse_roam_v1(void *buff, ssize_t buff_len)
+static void batctl_tvlv_parse_roam_v1(void *buff, ssize_t buff_len,
+				      int read_opt __maybe_unused)
 {
 	struct batadv_tvlv_roam_adv *tvlv = buff;
 
@@ -199,7 +206,8 @@ static void batctl_tvlv_parse_roam_v1(void *buff, ssize_t buff_len)
 }
 
 static void batctl_tvlv_parse_mcast_v1(void *buff __maybe_unused,
-				       ssize_t buff_len)
+				       ssize_t buff_len,
+				       int read_opt __maybe_unused)
 {
 	struct batadv_tvlv_mcast_data *tvlv = buff;
 	uint8_t flags;
@@ -218,7 +226,8 @@ static void batctl_tvlv_parse_mcast_v1(void *buff __maybe_unused,
 	       flags & BATADV_MCAST_WANT_ALL_IPV6 ? '6' : '.');
 }
 
-static void batctl_tvlv_parse_mcast_v2(void *buff, ssize_t buff_len)
+static void batctl_tvlv_parse_mcast_v2(void *buff, ssize_t buff_len,
+				       int read_opt __maybe_unused)
 {
 	struct batadv_tvlv_mcast_data *tvlv = buff;
 	uint8_t flags;
@@ -231,15 +240,46 @@ static void batctl_tvlv_parse_mcast_v2(void *buff, ssize_t buff_len)
 
 	flags = tvlv->flags;
 
-	printf("\tTVLV MCASTv2: [%c%c%c%s%s]\n",
+	printf("\tTVLV MCASTv2: [%c%c%c%s%s%c]\n",
 	       flags & BATADV_MCAST_WANT_ALL_UNSNOOPABLES ? 'U' : '.',
 	       flags & BATADV_MCAST_WANT_ALL_IPV4 ? '4' : '.',
 	       flags & BATADV_MCAST_WANT_ALL_IPV6 ? '6' : '.',
 	       !(flags & BATADV_MCAST_WANT_NO_RTR4) ? "R4" : ". ",
-	       !(flags & BATADV_MCAST_WANT_NO_RTR6) ? "R6" : ". ");
+	       !(flags & BATADV_MCAST_WANT_NO_RTR6) ? "R6" : ". ",
+	       !(flags & BATADV_MCAST_HAVE_MC_PTYPE_CAPA) ? 'P' : '.');
 }
 
-typedef void (*batctl_tvlv_parser_t)(void *buff, ssize_t buff_len);
+static void
+batctl_tvlv_parse_mcast_tracker_v1(void *buff, ssize_t buff_len, int read_opt)
+{
+	struct batadv_tvlv_mcast_tracker *tvlv = buff;
+	size_t tvlv_len = sizeof(*tvlv);
+	struct ether_addr *dst;
+	uint16_t num_dests;
+
+	if (buff_len < (ssize_t)tvlv_len) {
+		fprintf(stderr, "Warning - dropping received %s packet as it is not the correct size (%zu): %zu\n",
+			"TVLV MCAST TRACKER v1", tvlv_len, buff_len);
+		return;
+	}
+
+	num_dests = ntohs(tvlv->num_dests);
+	tvlv_len += num_dests * ETH_ALEN;
+	dst = (struct ether_addr *)(tvlv + 1);
+
+	if (buff_len < (ssize_t)tvlv_len) {
+		fprintf(stderr, "Warning - dropping received %s packet as it is not the correct size (%zu): %zu\n",
+			"TVLV MCAST TRACKER v1 (with destinations)", tvlv_len, buff_len);
+		return;
+	}
+
+	printf("\tTVLV MCAST TRACKER v1, destinations (%hu):\n", num_dests);
+
+	for (int i = 0; i < num_dests; i++)
+		printf("\t\t%s\n", get_name_by_macaddr(dst++, read_opt));
+}
+
+typedef void (*batctl_tvlv_parser_t)(void *buff, ssize_t buff_len, int read_opt);
 
 static batctl_tvlv_parser_t tvlv_parser_get(uint8_t type, uint8_t version)
 {
@@ -294,12 +334,20 @@ static batctl_tvlv_parser_t tvlv_parser_get(uint8_t type, uint8_t version)
 			return NULL;
 		}
 
+	case BATADV_TVLV_MCAST_TRACKER:
+		switch (version) {
+		case 1:
+			return batctl_tvlv_parse_mcast_tracker_v1;
+		default:
+			return NULL;
+		}
+
 	default:
 		return NULL;
 	}
 }
 
-static void dump_tvlv(unsigned char *ptr, ssize_t tvlv_len)
+static void dump_tvlv(unsigned char *ptr, ssize_t tvlv_len, int read_opt)
 {
 	struct batadv_tvlv_hdr *tvlv_hdr;
 	batctl_tvlv_parser_t parser;
@@ -317,7 +365,7 @@ static void dump_tvlv(unsigned char *ptr, ssize_t tvlv_len)
 
 		parser = tvlv_parser_get(tvlv_hdr->type, tvlv_hdr->version);
 		if (parser)
-			parser(ptr, len);
+			parser(ptr, len, read_opt);
 
 		/* go to the next container */
 		ptr += len;
@@ -357,7 +405,7 @@ static void dump_batman_ucast_tvlv(unsigned char *packet_buff, ssize_t buff_len,
 	       buff_len - sizeof(struct ether_header), tvlv_len,
 	       tvlv_packet->ttl);
 
-	dump_tvlv((uint8_t *)(tvlv_packet + 1), tvlv_len);
+	dump_tvlv((uint8_t *)(tvlv_packet + 1), tvlv_len, read_opt);
 }
 
 static int dump_bla2_claim(struct ether_header *eth_hdr,
@@ -604,6 +652,8 @@ static void dump_ipv6(unsigned char *packet_buff, ssize_t buff_len,
 			       (size_t)buff_len - sizeof(struct icmp6_hdr));
 			break;
 		case ND_NEIGHBOR_SOLICIT:
+			LEN_CHECK((size_t)buff_len - (size_t)(sizeof(struct ip6_hdr)),
+				  sizeof(*nd_neigh_sol), "ICMPv6 Neighbor Solicitation");
 			nd_neigh_sol = (struct nd_neighbor_solicit *)icmphdr;
 			inet_ntop(AF_INET6, &(nd_neigh_sol->nd_ns_target),
 				  nd_nas_target, 40);
@@ -611,6 +661,8 @@ static void dump_ipv6(unsigned char *packet_buff, ssize_t buff_len,
 			       nd_nas_target, buff_len);
 			break;
 		case ND_NEIGHBOR_ADVERT:
+			LEN_CHECK((size_t)buff_len - (size_t)(sizeof(struct ip6_hdr)),
+				  sizeof(*nd_advert), "ICMPv6 Neighbor Advertisement");
 			nd_advert = (struct nd_neighbor_advert *)icmphdr;
 			inet_ntop(AF_INET6, &(nd_advert->nd_na_target),
 				  nd_nas_target, 40);
@@ -646,7 +698,9 @@ static void dump_ip(unsigned char *packet_buff, ssize_t buff_len,
 	struct icmphdr *icmphdr;
 
 	iphdr = (struct iphdr *)packet_buff;
+	LEN_CHECK((size_t)buff_len, sizeof(*iphdr), ip_string);
 	LEN_CHECK((size_t)buff_len, (size_t)(iphdr->ihl * 4), ip_string);
+	LEN_CHECK((size_t)(iphdr->ihl * 4), sizeof(*iphdr), ip_string);
 
 	if (!time_printed)
 		print_time();
@@ -676,12 +730,20 @@ static void dump_ip(unsigned char *packet_buff, ssize_t buff_len,
 				(size_t)buff_len - (iphdr->ihl * 4));
 			break;
 		case ICMP_DEST_UNREACH:
-			LEN_CHECK((size_t)buff_len - (iphdr->ihl * 4) - sizeof(struct icmphdr),
-				sizeof(struct iphdr) + 8, "ICMP DEST_UNREACH");
-
 			switch (icmphdr->code) {
 			case ICMP_PORT_UNREACH:
+				LEN_CHECK((size_t)buff_len - (iphdr->ihl * 4) - sizeof(struct icmphdr),
+					  sizeof(struct iphdr), "ICMP DEST_UNREACH");
+
+				/* validate inner IP header information */
 				tmp_iphdr = (struct iphdr *)(((char *)icmphdr) + sizeof(struct icmphdr));
+				LEN_CHECK((size_t)buff_len - (iphdr->ihl * 4) - sizeof(struct icmphdr),
+					  (size_t)(tmp_iphdr->ihl * 4), "ICMP DEST_UNREACH");
+				LEN_CHECK((size_t)(tmp_iphdr->ihl * 4), sizeof(*iphdr), "ICMP DEST_UNREACH");
+
+				LEN_CHECK((size_t)buff_len - (iphdr->ihl * 4) - sizeof(struct icmphdr) - (tmp_iphdr->ihl * 4),
+					  sizeof(*tmp_udphdr), "ICMP DEST_UNREACH");
+
 				tmp_udphdr = (struct udphdr *)(((char *)tmp_iphdr) + (tmp_iphdr->ihl * 4));
 
 				printf("%s: ICMP ", ipdst);
@@ -778,7 +840,7 @@ static void dump_batman_iv_ogm(unsigned char *packet_buff, ssize_t buff_len, int
 	check_len -= sizeof(struct batadv_ogm_packet);
 	LEN_CHECK(check_len, (size_t)tvlv_len, "BAT OGM TVLV (containers)");
 
-	dump_tvlv((uint8_t *)(batman_ogm_packet + 1), tvlv_len);
+	dump_tvlv((uint8_t *)(batman_ogm_packet + 1), tvlv_len, read_opt);
 }
 
 static void dump_batman_ogm2(unsigned char *packet_buff, ssize_t buff_len,
@@ -822,7 +884,7 @@ static void dump_batman_ogm2(unsigned char *packet_buff, ssize_t buff_len,
 	check_len -= BATADV_OGM2_HLEN;
 	LEN_CHECK(check_len, (size_t)tvlv_len, "BAT OGM2 TVLV (containers)");
 
-	dump_tvlv((uint8_t *)(batman_ogm2 + 1), tvlv_len);
+	dump_tvlv((uint8_t *)(batman_ogm2 + 1), tvlv_len, read_opt);
 }
 
 static void dump_batman_elp(unsigned char *packet_buff, ssize_t buff_len,
@@ -863,7 +925,6 @@ static void dump_batman_icmp(unsigned char *packet_buff, ssize_t buff_len, int r
 	LEN_CHECK((size_t)buff_len - sizeof(struct ether_header), sizeof(struct batadv_icmp_packet), "BAT ICMP");
 
 	icmp_packet = (struct batadv_icmp_packet *)(packet_buff + sizeof(struct ether_header));
-	tp = (struct batadv_icmp_tp_packet *)icmp_packet;
 
 	if (!time_printed)
 		print_time();
@@ -894,6 +955,10 @@ static void dump_batman_icmp(unsigned char *packet_buff, ssize_t buff_len, int r
 			(size_t)buff_len - sizeof(struct ether_header));
 		break;
 	case BATADV_TP:
+		LEN_CHECK((size_t)buff_len - sizeof(struct ether_header), sizeof(*tp), "BAT TP");
+
+		tp = (struct batadv_icmp_tp_packet *)icmp_packet;
+
 		printf("%s: ICMP TP type %s (%hhu), id %hhu, seq %u, ttl %2d, v %d, length %zu\n",
 		       name, tp->subtype == BATADV_TP_MSG ? "MSG" :
 			     tp->subtype == BATADV_TP_ACK ? "ACK" : "N/A",
@@ -987,6 +1052,57 @@ static void dump_batman_bcast(unsigned char *packet_buff, ssize_t buff_len, int
 		      read_opt, time_printed);
 }
 
+static void dump_batman_mcast(unsigned char *packet_buff, ssize_t buff_len,
+			      int read_opt, int time_printed)
+{
+	struct batadv_mcast_packet *mcast_packet;
+	struct ether_header *ether_header;
+	uint8_t *tvlv_hdr;
+	ssize_t check_len;
+	size_t tvlv_len;
+
+	check_len = (size_t)buff_len - sizeof(struct ether_header);
+
+	LEN_CHECK(check_len, sizeof(*mcast_packet), "BAT MCAST");
+	check_len -= sizeof(*mcast_packet);
+
+	ether_header = (struct ether_header *)packet_buff;
+	mcast_packet = (struct batadv_mcast_packet *)(ether_header + 1);
+	tvlv_len = ntohs(mcast_packet->tvlv_len);
+
+	LEN_CHECK(check_len, tvlv_len, "BAT MCAST TVLV (containers)");
+	check_len -= tvlv_len;
+
+	tvlv_hdr = (uint8_t *)(mcast_packet + 1);
+
+	if (!time_printed)
+		time_printed = print_time();
+
+	printf("BAT %s > ",
+	       get_name_by_macaddr((struct ether_addr *)ether_header->ether_shost, read_opt));
+
+	printf("%s: MCAST, ttl %hhu, ",
+	       get_name_by_macaddr((struct ether_addr *)ether_header->ether_dhost, read_opt),
+	       mcast_packet->ttl);
+
+	/* batman-adv mcast packet's data payload is optional */
+	if (check_len >= ETH_HLEN) {
+		ether_header = (struct ether_header *)(tvlv_hdr + tvlv_len);
+
+		printf("%s > ",
+		       get_name_by_macaddr((struct ether_addr *)ether_header->ether_shost, read_opt));
+		printf("%s, ",
+		       get_name_by_macaddr((struct ether_addr *)ether_header->ether_dhost, read_opt));
+
+		parse_eth_hdr((unsigned char *)ether_header, check_len,
+			      read_opt, time_printed);
+	} else {
+		printf("\n");
+	}
+
+	dump_tvlv(tvlv_hdr, tvlv_len, read_opt);
+}
+
 static void dump_batman_coded(unsigned char *packet_buff, ssize_t buff_len, int read_opt, int time_printed)
 {
 	struct batadv_coded_packet *coded_packet;
@@ -1068,6 +1184,9 @@ static void parse_eth_hdr(unsigned char *packet_buff, ssize_t buff_len, int read
 			dump_vlan(packet_buff, buff_len, read_opt, time_printed);
 		break;
 	case ETH_P_BATMAN:
+		/* check for batman-adv packet_type + version */
+		LEN_CHECK(buff_len, sizeof(*eth_hdr) + 2, "BAT HEADER")
+
 		batman_ogm_packet = (struct batadv_ogm_packet *)(packet_buff + ETH_HLEN);
 
 		if ((read_opt & COMPAT_FILTER) &&
@@ -1105,6 +1224,10 @@ static void parse_eth_hdr(unsigned char *packet_buff, ssize_t buff_len, int read
 			if (dump_level & DUMP_TYPE_BATBCAST)
 				dump_batman_bcast(packet_buff, buff_len, read_opt, time_printed);
 			break;
+		case BATADV_MCAST:
+			if (dump_level & DUMP_TYPE_BATMCAST)
+				dump_batman_mcast(packet_buff, buff_len, read_opt, time_printed);
+			break;
 		case BATADV_CODED:
 			if (dump_level & DUMP_TYPE_BATCODED)
 				dump_batman_coded(packet_buff, buff_len, read_opt, time_printed);
